--- The utils module
-- @author Oskar Eisemuth
-- @copyright 2019
-- @module commonapi.events
local events = {
	_callbacks = {},
	dotrace = false,
}

local _utils = commonapi.utils
local _log = _utils.logPrep("commonapi.events")

--- Adds a event for callback
-- @tparam string eventname eventname
-- @tparam function fn callback function
-- @tparam string uid uniqueid
function events.add(eventname, fn, uid)
	local callbacks = events._callbacks
	if (callbacks[eventname] == nil) then
		callbacks[eventname] = setmetatable({}, {__mode="kv"})
	end
	if (uid == nil) then
		uid = debug.getinfo(2).source
	end
	local list = callbacks[eventname]

	local len = #list
	local newindex = 1
	for i=1, len do
		if (list[i][2] ~= uid) then
			if (i ~= newindex) then
				list[newindex] = list[i]
				list[i] = nil
			end
			newindex = newindex + 1
		else
			list[i] = nil
		end
	end

	list[#list+1] = { fn, uid }
end

function events.fire(eventname, data)
	events._fireevent(eventname, data)
end

function events._fireevent(eventname, data)
	local callbacks = events._callbacks

	if (events.dotrace == true) then
--		_log(eventname, data)
		_log(eventname)
		commonapi.dmp(data)
	end

	if (callbacks[eventname]  == nil) then
		return
	end

	local tev = callbacks[eventname]
	for i, cbinfo in ipairs(tev) do
		cbinfo[1](data)
	end
end

function events._activate()
	if (commonapi._native.setEventCallback ~= nil) then
		commonapi._native.setEventCallback(events._fireevent)
		return true
	end
	return false
end


function events.traceOn()
	events.dotrace = true
	_log("Tracing", events.dotrace)
end
return events
